import re, subprocess, json, os
from smsgateway.sources.sms import command_list
from smsgateway.config import *
from smsgateway import sink_sms

import logging
from logging.handlers import RotatingFileHandler
log_formatter = logging.Formatter('%(asctime)s %(levelname)s %(funcName)s(%(lineno)d) %(message)s')
logFile = os.path.join(LOG_DIR, 'send_signal.log')
my_handler = RotatingFileHandler(logFile, mode='a', maxBytes=5*1024*1024,
                                 backupCount=2, encoding=None, delay=0)
my_handler.setFormatter(log_formatter)
my_handler.setLevel(logging.INFO)
app_log = logging.getLogger('root')
app_log.setLevel(logging.INFO)
app_log.addHandler(my_handler)

command_regex = re.compile('^(?P<command>[a-zA-Z ]+)$')


def check(cmd, multiline):
    # app_log.info("Checking %s" % cmd)
    if cmd.lower() == 'sg' and multiline:
      return True
    else:
      return False


def run(lines):
    app_log.info("Forwarding Signal Message")
    toL = lines[1]
    m = re.match("To:? (.*)$", toL)
    if m:
      to = m.group(1).replace(' ', '')
      app_log.info("Matched To: %s" % to)
      msg = '\n'.join(lines[2:])
      app_log.info("Sending Signal msg:\n%s" % msg)
      try:
          args = [SIGNAL_CLI_PATH, '-u', SIGNAL_NUMBER, 'send', '-m', msg, to]
          app_log.info("Calling %s" % str(args))
          res = subprocess.call(args, timeout=30000)
      except subprocess.TimeoutExpired as e:
          msg = "Timeout when sending SG to %s: %s" % (to, e)
          app_log.error(msg)
          return msg
      except subprocess.CalledProcessError as e:
          msg = "Failed to send SG to %s: %s" % (to, e.output)
          app_log.error(msg)
          return msg
      if res == 0:
          ret = "Sent SG to %s!" % to
      else:
          ret = "Failed to send SG to %s!" % to
    else:
      ret = "Couldn't match To: %s" % toL
      app_log.info(ret)
    return ret

command_list.append({
    'name' : 'SG-Forwarder',
    'check': check,
    'run': run
})



#!/usr/bin/python3
import os, argparse, re, subprocess, time
import importlib
from smsgateway import sink_sms
from smsgateway.config import *
from smsgateway.sources.utils import *

app_log = setup_logging("telegram")

src_path = os.path.dirname(os.path.abspath(__file__))

command_list = []
mods = []

for file in os.listdir(os.path.join(src_path, 'commands')):
      ext_file = os.path.splitext(file)

      if ext_file[1] == '.py' and not ext_file[0] == '__init__':
          app_log.debug("Importing %s" % ext_file[0])
          m = importlib.import_module('smsgateway.sources.commands.' + ext_file[0])
          mods.append(m)

def handleCommand(text):
  lines = text.strip().split('\n')
  cmd = lines[0].lower().strip()
  ret = "Unknown Command:\n%s" % text
  to = CONTROL_PHONES[0]
  app_log.info("Message has %s lines" % len(lines))
  #if len(lines) == 1:
  app_log.info("Commands: %s" % str(mods))
  multiline = len(lines) > 1
  for c in mods:
      if c.check(cmd, multiline):
          print(f"Mod {c} matched :)")
          try:
              ret = c.run(lines)
          except Exception as e:
              sink_sms.send_notif("Run failed for command %s:\n%s" % (cmd, e))
          break
  # elif len(lines) > 2:
  #   if cmd == "SMS":
  #     print("Forwarding SMS")
  #     toL = lines[1]
  #     m = re.match("To: ([0-9 +]+)", toL)
  #     if m:
  #       to = m.group(1).replace(' ', '')
  #       print("Matched To: %s" % to)
  #       ret = '\n'.join(lines[2:])
  #     else:
  #       print("Couldn't match To: %s" % to)
  if ret:
    sink_sms.send_to(to, ret)

def handleSMS(f):
    app_log.info("Received SMS, file: %s" % f)
    From = None
    textStarted = False
    decodeUCS2 = False
    text = ''
    with open(f, 'rb') as f:
      app_log.info("Opened received SMS")
      lines = f.readlines()
      for line in lines:
        if textStarted == True and decodeUCS2:
          line = line.decode("utf-16-be")
        else:
          line = line.decode("iso8859-15")

        line = line.rstrip()
        m = re.match("From: ([0-9]+)", line)
        m2 = re.match("From: ([A-Za-z0-9]+)", line)
        mAlpha = re.match("Alphabet: ([A-Z0-9]+)", line)
        if textStarted == True:
            text += "%s\n" % line
        elif m or m2:
            if m:
              From = "+%s" % m.group(1)
            else:
              From = "%s" % m2.group(1)
              app_log.info("Number is not numeric!")
            app_log.info("From: %s" % From)
        elif mAlpha:
            alpha = mAlpha.group(1)
            if alpha == "UCS2":
                decodeUCS2 = True
            elif alpha == "ISO":
                decodeUCS2 = False
        elif line == "":
            textStarted = True

    if From and textStarted:
      if From in ['+%s' % num for num in CONTROL_PHONES]:
        handleCommand(text)
      else: # SMS from someone else
        app_log.info("Sending SMS")
        sink_sms.send("SMS", text, From)
    else:
      app_log.error("Couldn't parse incoming sms!")

def resendSMS(f):
  textStarted = False
  To = None
  textStarted = None
  wholeText = ''
  text = ''
  with open(f, 'r', encoding="8859") as f:
    app_log.info("Opened failed SMS")
    for line in f:
      line = line.rstrip()
      wholeText += line
      if textStarted == True:
        text += "%s\n" % line
      else:
        m = re.match("To: ([0-9]+)", line)
        m_fail = re.match("Fail_reason: (.*)", line)
        if m:
          To = "+%s" % m.group(1)
        elif m_fail:
          fail_reason = m_fail.group(1)
          app_log.warning("Fail reason: %s" % fail_reason)
        elif line == "":
          textStarted = True
    if To and textStarted:
      if To in CONTROL_PHONES:
        app_log.warning("Resending SMS to CONTROL_PHONE")
        new_text = "RESEND: %s\n%s" % (fail_reason, text)
        sink_sms.send_notif(new_text)
      else:
        app_log.warning("Resending SMS to %s" % To)
        sink_sms.send_to(To, text)
        new_text = "RESEND: %s\n%s" % (fail_reason, text)
        sink_sms.send_notif(new_text)
    else:
      new_text = "Invalid failed SMS:\n%s" % wholeText
      app_log.error(new_text)
      sink_sms.send_notif(new_text)

def main():
    app_log.info(src_path)
    parser = argparse.ArgumentParser()
    parser.add_argument("event")
    parser.add_argument("file")
    args = parser.parse_args()

    if not args.event:
      app_log.error("Unknown command, use -h for help")
    else:
      if args.event == "RECEIVED":
        handleSMS(args.file)
      elif args.event == 'FAILED':
        if args.file:
          app_log.warning("SMS failed. Process:")
          app_log.warning("- Stopping smstools.. ", end='')
          (result, out) = run_cmd([SUDO_PATH, SYSTEMCTL_PATH, 'stop', 'smstools'])
          if result == 0:
            app_log.info("Success!")
          else:
            app_log.error("Error:\n%s" % out)

          app_log.info("- Resending: %s" % args.file)
          resendSMS(args.file)
          app_log.warning("- Reboot!", flush=True)
          app_log.warning('')
          (res, out) = run_cmd([SUDO_PATH, REBOOT_PATH], "Reboot")
          if res != 0:
            app_log.error("Reboot failed: %s" % out)
        else:
          app_log.error("Got FAILED event, but no file!?")
      else:
        app_log.error("Event is %s, expected RECEIVED" % args.event)

if __name__ == '__main__':
    main()



from smsgateway import sink_sms
from smsgateway.sources.utils import *
from smsgateway.config import *

from imapclient import IMAPClient
import email, time
from email.header import decode_header, make_header
from bs4 import BeautifulSoup

app_log = setup_logging("email")

IDENTIFIER = "EM"

def parse_fetch(fetch):
    msg = email.message_from_bytes(fetch)
    body = None
    if msg.is_multipart():
        for part in msg.get_payload():
            if part.get_content_maintype() == 'text':
                body = part.get_payload(decode=True).decode('UTF-8')
    else:
        body = msg.get_payload(decode=True).decode('UTF-8')
        type = msg.get_content_type()
        if type == 'text/html':
            # Extract the actual text content
            b = BeautifulSoup(body, 'html.parser')
            for s in b(["script", "style"]):
                s.decompose()
            text = b.get_text()
            lines = (line.strip() for line in text.splitlines())
            chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
            text = '\n'.join(chunk for chunk in chunks if chunk)
            body = text
    items = {'Body': body}
    for name in ['Subject', 'From', 'To']:
        items[name] = make_header(decode_header(msg[name]))

    return items
def fetch_messages(server, num):
    app_log.info("Fetching %s messages" % num)
    msg_list = server.sort(sort_criteria='REVERSE DATE', charset='UTF-8')
    app_log.debug("Found %s messages" % len(msg_list))
    last_messages = msg_list[0:num]
    msg_fetches = server.fetch(last_messages, ['RFC822'])
    app_log.info("Fetched %s messages" % len(msg_fetches))
    messages = []
    for msg_id in last_messages:
        app_log.debug(msg_id)
        fetch = msg_fetches[msg_id]
        if b'RFC822' in fetch:
            msg = parse_fetch(fetch[b'RFC822'])
            messages.append(msg)
            for k, v in msg.items():
                app_log.debug("%s: %s" % (k, v))
        else:
            app_log.warning("No RFC822 in this fetch: %s" % fetch)
    return messages

def login(server):
    server.login(EMAIL_USERNAME, EMAIL_PASSWORD)
    inbox = server.select_folder('INBOX')
    app_log.debug("Inbox: %s" % inbox)

def wait_idle(server):
    app_log.info("Waiting for IDLE response..")
    server.idle()
    last_exists = 0
    while True:
        # noop = server.noop()
        # app_log.debug("Server returned %s" % noop)
        app_log.debug("Checking IDLE")
        start = time.time()
        responses = server.idle_check(timeout=30)
        diff = time.time() - start
        app_log.debug("Server sent: %s" % responses if responses else "nothing")
        if responses:
            num = 0
            for resp in responses:
              if len(resp) >= 2:
                n = resp[0]
                type = resp[1]
                if type == b'EXISTS':
                  if n > last_exists:
                    num += 1
                  last_exists = n
            if num > 0:
              server.idle_done()
              messages = fetch_messages(server, num)
              for msg in messages:
                  body = msg['Body']
                  del msg['Body']
                  app_log.debug("Forwarding message: %s" % str(msg))
                  sink_sms.send_dict(IDENTIFIER, body, msg)
              server.idle()
        elif diff < 5:
            app_log.warning(f"IDLE check took only {diff} seconds!\nNeed to reconnect..")
            try:
                server.logout()
            except Exception as e:
                app_log.warning(f"Couldn't log out: {e}")
            login(server)

    server.idle_done()

def main():
    app_log.info("Logging in..")
    server = IMAPClient(EMAIL_HOST)
    login(server)

    wait_idle(server)

    server.logout()

if __name__ == '__main__':
    main()
